@@ -204,6 +204,7 @@
 #define HAVE_DL_ITERATE_PHDR 1
 #define HAVE_DLOPEN 1
 #define HAVE_DUP 1
+#define HAVE_DUP3 1
 #define HAVE_ENDGRENT 1
 #define HAVE_FCHMOD 1
 #define HAVE_FCHOWN 1
@@ -237,6 +238,7 @@
 #define HAVE_MBLEN 1
 #define HAVE_MEMRCHR 1
 #define HAVE_MKTIME 1
+#define HAVE_PIPE2 1
 #define HAVE_POLL 1
 #define HAVE_POSIX_MEMALIGN 1
 #define HAVE_PPOLL 1
@@ -282,13 +284,14 @@
 #define HAVE_BUILTIN___BUILTIN_CHOOSE_EXPR 1
 #define HAVE_BUILTIN___BUILTIN_TYPES_COMPATIBLE_P 1
 #define HAVE_CLOCK_GETRES 1
-#define RUBY_SETJMP(env) _setjmp(env)
-#define RUBY_LONGJMP(env,val) _longjmp(env,val)
-#define RUBY_JMP_BUF jmp_buf
 #define HAVE_STRUCT_TM_TM_ZONE 1
 #define HAVE_TM_ZONE 1
 #define HAVE_STRUCT_TM_TM_GMTOFF 1
+#define HAVE_DAYLIGHT 1
+#define HAVE_VAR_TIMEZONE 1
+#define TYPEOF_VAR_TIMEZONE long
 #define HAVE_TIMEZONE 1
+#define TIMEZONE_VOID 1
 #define NEGATIVE_TIME_T 1
 #define POSIX_SIGNAL 1
 #define RSHIFT(x,y) ((x)>>(int)(y))
@@ -315,6 +318,9 @@
 #define DLEXT_MAXLEN 3
 #define DLEXT ".so"
 #define LIBDIR_BASENAME "lib"
+#define RUBY_SETJMP(env) _setjmp((env))
+#define RUBY_LONGJMP(env,val) _longjmp((env),val)
+#define RUBY_JMP_BUF jmp_buf
 #define HAVE_PTHREAD_H 1
 #define HAVE_MEMMEM 1
 #define RUBY_PLATFORM "arm-openbsd"
