@@ -1,34 +1,36 @@
 #
-#   ruby-lex.rb - ruby lexcal analizer
-#   	$Release Version: 0.6$
-#   	$Revision: 1.20 $
-#   	$Date: 1999/03/11 15:39:55 $
-#   	by Keiju ISHITSUKA(Nippon Rational Inc.)
+#   irb/ruby-lex.rb - ruby lexcal analyzer
+#   	$Release Version: 0.9.6$
+#   	$Revision: 38632 $
+#   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
 #
-#   
+#
 #
 
 require "e2mmap"
 require "irb/slex"
 require "irb/ruby-token"
 
+# :stopdoc:
 class RubyLex
-  @RCS_ID='-$Id: ruby-lex.rb 1.20 1999/03/11 15:39:55 keiju Exp keiju $-'
+  @RCS_ID='-$Id: ruby-lex.rb 38632 2012-12-27 14:48:42Z keiju $-'
 
   extend Exception2MessageMapper
   def_exception(:AlreadyDefinedToken, "Already defined token(%s)")
   def_exception(:TkReading2TokenNoKey, "key nothing(key='%s')")
   def_exception(:TkSymbol2TokenNoKey, "key nothing(key='%s')")
-  def_exception(:TkReading2TokenDuplicateError, 
+  def_exception(:TkReading2TokenDuplicateError,
 		"key duplicate(token_n='%s', key='%s')")
   def_exception(:SyntaxError, "%s")
-  
+
+  def_exception(:TerminateLineInput, "Terminate Line Input")
+
   include RubyToken
 
   class << self
-    attr :debug_level, TRUE
+    attr_accessor :debug_level
     def debug?
       @debug_level > 0
     end
@@ -48,40 +50,50 @@ class RubyLex
     @here_readed = []
 
     @indent = 0
+    @indent_stack = []
+    @lex_state = EXPR_BEG
+    @space_seen = false
+    @here_header = false
+    @post_symbeg = false
+
+    @continue = false
+    @line = ""
 
     @skip_space = false
     @readed_auto_clean_up = false
     @exception_on_syntax_error = true
+
+    @prompt = nil
   end
 
-  attr :skip_space, true
-  attr :readed_auto_clean_up, true
-  attr :exception_on_syntax_error, true
+  attr_accessor :skip_space
+  attr_accessor :readed_auto_clean_up
+  attr_accessor :exception_on_syntax_error
 
-  attr :seek
-  attr :char_no
-  attr :line_no
-  attr :indent
+  attr_reader :seek
+  attr_reader :char_no
+  attr_reader :line_no
+  attr_reader :indent
 
   # io functions
-  def set_input(io, p = nil)
+  def set_input(io, p = nil, &block)
     @io = io
-    if p.kind_of?(Proc)
+    if p.respond_to?(:call)
       @input = p
-    elsif iterator?
-      @input = proc
+    elsif block_given?
+      @input = block
     else
-      @input = proc{@io.gets}
+      @input = Proc.new{@io.gets}
     end
   end
 
   def get_readed
-    if idx = @readed.reverse.index("\n")
-      @base_char_no = idx
+    if idx = @readed.rindex("\n")
+      @base_char_no = @readed.size - (idx + 1)
     else
       @base_char_no += @readed.size
     end
-    
+
     readed = @readed.join("")
     @readed = []
     readed
@@ -89,7 +101,8 @@ class RubyLex
 
   def getc
     while @rests.empty?
-      return nil unless buf_input
+#      return nil unless buf_input
+      @rests.push nil unless buf_input
     end
     c = @rests.shift
     if @here_header
@@ -99,7 +112,7 @@ class RubyLex
     end
     @seek += 1
     if c == "\n"
-      @line_no += 1 
+      @line_no += 1
       @char_no = 0
     else
       @char_no += 1
@@ -110,9 +123,10 @@ class RubyLex
   def gets
     l = ""
     while c = getc
-      l.concat c
+      l.concat(c)
       break if c == "\n"
     end
+    return nil if l == "" and c.nil?
     l
   end
 
@@ -135,12 +149,12 @@ class RubyLex
       c2 = @here_readed.pop
     end
     c = c2 unless c
-    @rests.unshift c #c = 
-      @seek -= 1
+    @rests.unshift c #c =
+    @seek -= 1
     if c == "\n"
-      @line_no -= 1 
-      if idx = @readed.reverse.index("\n")
-	@char_no = @readed.size - idx
+      @line_no -= 1
+      if idx = @readed.rindex("\n")
+	@char_no = idx + 1
       else
 	@char_no = @base_char_no + @readed.size
       end
@@ -175,16 +189,17 @@ class RubyLex
     prompt
     line = @input.call
     return nil unless line
-    @rests.concat line.split(//)
+    @rests.concat line.chars.to_a
     true
   end
   private :buf_input
 
-  def set_prompt(p = proc)
-    if p.kind_of?(Proc)
+  def set_prompt(p = nil, &block)
+    p = block if block_given?
+    if p.respond_to?(:call)
       @prompt = p
     else
-      @prompt = proc{print p}
+      @prompt = Proc.new{print p}
     end
   end
 
@@ -198,40 +213,52 @@ class RubyLex
     @ltype = nil
     @quoted = nil
     @indent = 0
+    @indent_stack = []
     @lex_state = EXPR_BEG
     @space_seen = false
     @here_header = false
-    
+
+    @continue = false
+    @post_symbeg = false
+
     prompt
-    @continue = FALSE
 
     @line = ""
     @exp_line_no = @line_no
   end
-  
+
   def each_top_level_statement
     initialize_input
-    loop do
-      @continue = FALSE
-      prompt
-      unless l = lex
-	break if @line == ''
-      else
-	# p l
-	@line.concat l
-	if @ltype or @continue or @indent > 0
-	  next
+    catch(:TERM_INPUT) do
+      loop do
+	begin
+	  @continue = false
+	  prompt
+	  unless l = lex
+	    throw :TERM_INPUT if @line == ''
+	  else
+	    @line.concat l
+	    if @ltype or @continue or @indent > 0
+	      next
+	    end
+	  end
+	  if @line != "\n"
+            @line.force_encoding(@io.encoding)
+	    yield @line, @exp_line_no
+	  end
+	  break unless l
+	  @line = ''
+	  @exp_line_no = @line_no
+
+	  @indent = 0
+	  @indent_stack = []
+	  prompt
+	rescue TerminateLineInput
+	  initialize_input
+	  prompt
+	  get_readed
 	end
       end
-      if @line != "\n"
-	yield @line, @exp_line_no
-      end
-      break unless l
-      @line = ''
-      @exp_line_no = @line_no
-
-      @indent = 0
-      prompt
     end
   end
 
@@ -239,8 +266,9 @@ class RubyLex
     until (((tk = token).kind_of?(TkNL) || tk.kind_of?(TkEND_OF_SCRIPT)) &&
 	     !@continue or
 	     tk.nil?)
-      #	p tk
-      #	p self
+      #p tk
+      #p @lex_state
+      #p self
     end
     line = get_readed
     #      print self.inspect
@@ -261,8 +289,10 @@ class RubyLex
       begin
 	tk = @OP.match(self)
 	@space_seen = tk.kind_of?(TkSPACE)
+	@lex_state = EXPR_END if @post_symbeg && tk.kind_of?(TkOp)
+	@post_symbeg = tk.kind_of?(TkSYMBEG)
       rescue SyntaxError
-	abort if @exception_on_syntax_error
+	raise if @exception_on_syntax_error
 	tk = TkError.new(@seek, @line_no, @char_no)
       end
     end while @skip_space and tk.kind_of?(TkSPACE)
@@ -272,7 +302,7 @@ class RubyLex
     #      Tracer.off
     tk
   end
-  
+
   ENINDENT_CLAUSE = [
     "case", "class", "def", "do", "for", "if",
     "module", "unless", "until", "while", "begin" #, "when"
@@ -284,10 +314,14 @@ class RubyLex
     "q" => "\'",
     "Q" => "\"",
     "x" => "\`",
-    "r" => "\/",
-    "w" => "]"
+    "r" => "/",
+    "w" => "]",
+    "W" => "]",
+    "i" => "]",
+    "I" => "]",
+    "s" => ":"
   }
-  
+
   PERCENT_PAREN = {
     "{" => "}",
     "[" => "]",
@@ -299,80 +333,99 @@ class RubyLex
     "\'" => TkSTRING,
     "\"" => TkSTRING,
     "\`" => TkXSTRING,
-    "\/" => TkREGEXP,
-    "]" => TkDSTRING
+    "/" => TkREGEXP,
+    "]" => TkDSTRING,
+    ":" => TkSYMBOL
   }
   DLtype2Token = {
     "\"" => TkDSTRING,
     "\`" => TkDXSTRING,
-    "\/" => TkDREGEXP,
+    "/" => TkDREGEXP,
   }
 
   def lex_init()
-    @OP = SLex.new
-    @OP.def_rules("\0", "\004", "\032") do
+    @OP = IRB::SLex.new
+    @OP.def_rules("\0", "\004", "\032") do |op, io|
       Token(TkEND_OF_SCRIPT)
     end
 
-    @OP.def_rules(" ", "\t", "\f", "\r", "\13") do
-      @space_seen = TRUE
+    @OP.def_rules(" ", "\t", "\f", "\r", "\13") do |op, io|
+      @space_seen = true
       while getc =~ /[ \t\f\r\13]/; end
       ungetc
       Token(TkSPACE)
     end
 
-    @OP.def_rule("#") do
-      |op, io|
+    @OP.def_rule("#") do |op, io|
       identify_comment
     end
 
-    @OP.def_rule("=begin", proc{@prev_char_no == 0 && peek(0) =~ /\s/}) do
+    @OP.def_rule("=begin",
+		 proc{|op, io| @prev_char_no == 0 && peek(0) =~ /\s/}) do
       |op, io|
       @ltype = "="
       until getc == "\n"; end
       until peek_equal?("=end") && peek(4) =~ /\s/
 	until getc == "\n"; end
       end
-      getc; getc; getc; getc
+      gets
       @ltype = nil
       Token(TkRD_COMMENT)
     end
 
-    @OP.def_rule("\n") do
+    @OP.def_rule("\n") do |op, io|
       print "\\n\n" if RubyLex.debug?
       case @lex_state
       when EXPR_BEG, EXPR_FNAME, EXPR_DOT
-	@continue = TRUE
+	@continue = true
       else
-	@continue = FALSE
+	@continue = false
 	@lex_state = EXPR_BEG
+	until (@indent_stack.empty? ||
+	       [TkLPAREN, TkLBRACK, TkLBRACE,
+		 TkfLPAREN, TkfLBRACK, TkfLBRACE].include?(@indent_stack.last))
+	  @indent_stack.pop
+	end
       end
       @here_header = false
       @here_readed = []
       Token(TkNL)
     end
 
-    @OP.def_rules("*", "**",	
-		  "!", "!=", "!~",
-		  "=", "==", "===", 
-		  "=~", "<=>",	
+    @OP.def_rules("*", "**",
+		  "=", "==", "===",
+		  "=~", "<=>",
 		  "<", "<=",
-		  ">", ">=", ">>") do
+		  ">", ">=", ">>",
+		  "!", "!=", "!~") do
       |op, io|
-      @lex_state = EXPR_BEG
+      case @lex_state
+      when EXPR_FNAME, EXPR_DOT
+	@lex_state = EXPR_ARG
+      else
+	@lex_state = EXPR_BEG
+      end
       Token(op)
     end
 
     @OP.def_rules("<<") do
       |op, io|
+      tk = nil
       if @lex_state != EXPR_END && @lex_state != EXPR_CLASS &&
 	  (@lex_state != EXPR_ARG || @space_seen)
 	c = peek(0)
-	if /\S/ =~ c && (/["'`]/ =~ c || /[\w_]/ =~ c || c == "-")
+	if /\S/ =~ c && (/["'`]/ =~ c || /\w/ =~ c || c == "-")
 	  tk = identify_here_document
 	end
-      else
-	tk = 	Token(op)
+      end
+      unless tk
+	tk = Token(op)
+	case @lex_state
+	when EXPR_FNAME, EXPR_DOT
+	  @lex_state = EXPR_ARG
+	else
+	  @lex_state = EXPR_BEG
+	end
       end
       tk
     end
@@ -385,6 +438,7 @@ class RubyLex
     @OP.def_rules("`") do
       |op, io|
       if @lex_state == EXPR_FNAME
+	@lex_state = EXPR_END
 	Token(op)
       else
 	identify_string(op)
@@ -398,12 +452,12 @@ class RubyLex
 	Token(TkQUESTION)
       else
 	ch = getc
-	if @lex_state == EXPR_ARG && ch !~ /\s/
+	if @lex_state == EXPR_ARG && ch =~ /\s/
 	  ungetc
 	  @lex_state = EXPR_BEG;
 	  Token(TkQUESTION)
 	else
-	  if (ch == '\\') 
+	  if (ch == '\\')
 	    read_escape
 	  end
 	  @lex_state = EXPR_END
@@ -417,8 +471,8 @@ class RubyLex
       @lex_state = EXPR_BEG
       Token(op)
     end
-    
-    @OP.def_rules("+=", "-=", "*=", "**=", 
+
+    @OP.def_rules("+=", "-=", "*=", "**=",
 		  "&=", "|=", "^=", "<<=", ">>=", "||=", "&&=") do
       |op, io|
       @lex_state = EXPR_BEG
@@ -426,12 +480,16 @@ class RubyLex
       Token(TkOPASGN, $1)
     end
 
-    @OP.def_rule("+@", proc{@lex_state == EXPR_FNAME}) do
-      Token(TkUPLUS)
+    @OP.def_rule("+@", proc{|op, io| @lex_state == EXPR_FNAME}) do
+      |op, io|
+      @lex_state = EXPR_ARG
+      Token(op)
     end
 
-    @OP.def_rule("-@", proc{@lex_state == EXPR_FNAME}) do
-      Token(TkUMINUS)
+    @OP.def_rule("-@", proc{|op, io| @lex_state == EXPR_FNAME}) do
+      |op, io|
+      @lex_state = EXPR_ARG
+      Token(op)
     end
 
     @OP.def_rules("+", "-") do
@@ -453,12 +511,13 @@ class RubyLex
     end
 
     @OP.def_rule(".") do
+      |op, io|
       @lex_state = EXPR_BEG
       if peek(0) =~ /[0-9]/
 	ungetc
 	identify_number
       else
-	# for obj.if
+	# for "obj.if" etc.
 	@lex_state = EXPR_DOT
 	Token(TkDOT)
       end
@@ -472,26 +531,29 @@ class RubyLex
 
     lex_int2
   end
-  
+
   def lex_int2
     @OP.def_rules("]", "}", ")") do
       |op, io|
       @lex_state = EXPR_END
       @indent -= 1
+      @indent_stack.pop
       Token(op)
     end
 
     @OP.def_rule(":") do
+      |op, io|
       if @lex_state == EXPR_END || peek(0) =~ /\s/
 	@lex_state = EXPR_BEG
 	Token(TkCOLON)
       else
-	@lex_state = EXPR_FNAME;
+	@lex_state = EXPR_FNAME
 	Token(TkSYMBEG)
       end
     end
 
     @OP.def_rule("::") do
+       |op, io|
 #      p @lex_state.id2name, @space_seen
       if @lex_state == EXPR_BEG or @lex_state == EXPR_ARG && @space_seen
 	@lex_state = EXPR_BEG
@@ -509,16 +571,17 @@ class RubyLex
       elsif peek(0) == '='
 	getc
 	@lex_state = EXPR_BEG
-	Token(TkOPASGN, :/) #/)
+	Token(TkOPASGN, "/") #/)
       elsif @lex_state == EXPR_ARG and @space_seen and peek(0) !~ /\s/
 	identify_string(op)
-      else 
+      else
 	@lex_state = EXPR_BEG
 	Token("/") #/)
       end
     end
 
     @OP.def_rules("^") do
+      |op, io|
       @lex_state = EXPR_BEG
       Token("^")
     end
@@ -527,77 +590,102 @@ class RubyLex
     # 	@lex_state = EXPR_BEG
     # 	Token(OP_ASGN, :^)
     #       end
-    
-    @OP.def_rules(",", ";") do
+
+    @OP.def_rules(",") do
       |op, io|
       @lex_state = EXPR_BEG
       Token(op)
     end
 
+    @OP.def_rules(";") do
+      |op, io|
+      @lex_state = EXPR_BEG
+      until (@indent_stack.empty? ||
+	     [TkLPAREN, TkLBRACK, TkLBRACE,
+	       TkfLPAREN, TkfLBRACK, TkfLBRACE].include?(@indent_stack.last))
+	@indent_stack.pop
+      end
+      Token(op)
+    end
+
     @OP.def_rule("~") do
+      |op, io|
       @lex_state = EXPR_BEG
       Token("~")
     end
 
-    @OP.def_rule("~@", proc{@lex_state = EXPR_FNAME}) do
+    @OP.def_rule("~@", proc{|op, io| @lex_state == EXPR_FNAME}) do
+      |op, io|
       @lex_state = EXPR_BEG
       Token("~")
     end
-    
+
     @OP.def_rule("(") do
+      |op, io|
       @indent += 1
       if @lex_state == EXPR_BEG || @lex_state == EXPR_MID
 	@lex_state = EXPR_BEG
-	Token(TkfLPAREN)
+	tk_c = TkfLPAREN
       else
 	@lex_state = EXPR_BEG
-	Token(TkLPAREN)
+	tk_c = TkLPAREN
       end
+      @indent_stack.push tk_c
+      Token(tk_c)
     end
 
-    @OP.def_rule("[]", proc{@lex_state == EXPR_FNAME}) do
+    @OP.def_rule("[]", proc{|op, io| @lex_state == EXPR_FNAME}) do
+      |op, io|
+      @lex_state = EXPR_ARG
       Token("[]")
     end
 
-    @OP.def_rule("[]=", proc{@lex_state == EXPR_FNAME}) do
+    @OP.def_rule("[]=", proc{|op, io| @lex_state == EXPR_FNAME}) do
+      |op, io|
+      @lex_state = EXPR_ARG
       Token("[]=")
     end
 
     @OP.def_rule("[") do
+      |op, io|
       @indent += 1
       if @lex_state == EXPR_FNAME
-	Token(TkfLBRACK)
+	tk_c = TkfLBRACK
       else
 	if @lex_state == EXPR_BEG || @lex_state == EXPR_MID
-	  t = Token(TkLBRACK)
+	  tk_c = TkLBRACK
 	elsif @lex_state == EXPR_ARG && @space_seen
-	  t = Token(TkLBRACK)
+	  tk_c = TkLBRACK
 	else
-	  t = Token(TkfLBRACK)
+	  tk_c = TkfLBRACK
 	end
 	@lex_state = EXPR_BEG
-	t
       end
+      @indent_stack.push tk_c
+      Token(tk_c)
     end
 
     @OP.def_rule("{") do
+      |op, io|
       @indent += 1
       if @lex_state != EXPR_END && @lex_state != EXPR_ARG
-	t = Token(TkLBRACE)
+	tk_c = TkLBRACE
       else
-	t = Token(TkfLBRACE)
+	tk_c = TkfLBRACE
       end
       @lex_state = EXPR_BEG
-      t
+      @indent_stack.push tk_c
+      Token(tk_c)
     end
 
     @OP.def_rule('\\') do
+      |op, io|
       if getc == "\n"
 	@space_seen = true
 	@continue = true
 	Token(TkSPACE)
       else
-	ungetc
+	read_escape
 	Token("\\")
       end
     end
@@ -608,7 +696,7 @@ class RubyLex
 	identify_quotation
       elsif peek(0) == '='
 	getc
-	Token(OP_ASGIN, "%")
+	Token(TkOPASGN, :%)
       elsif @lex_state == EXPR_ARG and @space_seen and peek(0) !~ /\s/
 	identify_quotation
       else
@@ -618,11 +706,13 @@ class RubyLex
     end
 
     @OP.def_rule('$') do
+      |op, io|
       identify_gvar
     end
 
     @OP.def_rule('@') do
-      if peek(0) =~ /[\w_]/
+      |op, io|
+      if peek(0) =~ /[\w@]/
 	ungetc
 	identify_identifier
       else
@@ -630,7 +720,7 @@ class RubyLex
       end
     end
 
-    #       @OP.def_rule("def", proc{|op, io| /\s/ =~ io.peek(0)}) do 
+    #       @OP.def_rule("def", proc{|op, io| /\s/ =~ io.peek(0)}) do
     # 	|op, io|
     # 	@indent += 1
     # 	@lex_state = EXPR_FNAME
@@ -645,19 +735,19 @@ class RubyLex
       printf "MATCH: start %s: %s\n", op, io.inspect if RubyLex.debug?
       if peek(0) =~ /[0-9]/
 	t = identify_number
-      elsif peek(0) =~ /[\w_]/
+      elsif peek(0) =~ /[^\x00-\/:-@\[-^`{-\x7F]/
 	t = identify_identifier
       end
       printf "MATCH: end %s: %s\n", op, io.inspect if RubyLex.debug?
       t
     end
-    
+
     p @OP if RubyLex.debug?
   end
-  
+
   def identify_gvar
     @lex_state = EXPR_END
-    
+
     case ch = getc
     when /[~_*$?!@\/\\;,=:<>".]/   #"
       Token(TkGVAR, "$" + ch)
@@ -673,34 +763,45 @@ class RubyLex
       ungetc
       ungetc
       identify_identifier
-    else 
+    else
       ungetc
       Token("$")
     end
   end
-  
+
   def identify_identifier
     token = ""
-    token.concat getc if peek(0) =~ /[$@]/
-    while (ch = getc) =~ /\w|_/
+    if peek(0) =~ /[$@]/
+      token.concat(c = getc)
+      if c == "@" and peek(0) == "@"
+	token.concat getc
+      end
+    end
+
+    while (ch = getc) =~ /[^\x00-\/:-@\[-^`{-\x7F]/
       print ":", ch, ":" if RubyLex.debug?
       token.concat ch
     end
     ungetc
-    
-    if ch == "!" or ch == "?"
+
+    if (ch == "!" || ch == "?") && token[0,1] =~ /\w/ && peek(0) != "="
       token.concat getc
     end
-    # fix token
+
+    # almost fix token
 
     case token
     when /^\$/
       return Token(TkGVAR, token)
+    when /^\@\@/
+      @lex_state = EXPR_END
+      # p Token(TkCVAR, token)
+      return Token(TkCVAR, token)
     when /^\@/
       @lex_state = EXPR_END
       return Token(TkIVAR, token)
     end
-    
+
     if @lex_state != EXPR_DOT
       print token, "\n" if RubyLex.debug?
 
@@ -717,9 +818,36 @@ class RubyLex
 	else
 	  if @lex_state != EXPR_FNAME
 	    if ENINDENT_CLAUSE.include?(token)
-	      @indent += 1
+	      # check for ``class = val'' etc.
+	      valid = true
+	      case token
+	      when "class"
+		valid = false unless peek_match?(/^\s*(<<|\w|::)/)
+	      when "def"
+		valid = false if peek_match?(/^\s*(([+\-\/*&\|^]|<<|>>|\|\||\&\&)=|\&\&|\|\|)/)
+	      when "do"
+		valid = false if peek_match?(/^\s*([+\-\/*]?=|\*|<|>|\&)/)
+	      when *ENINDENT_CLAUSE
+		valid = false if peek_match?(/^\s*([+\-\/*]?=|\*|<|>|\&|\|)/)
+	      else
+		# no nothing
+	      end
+	      if valid
+		if token == "do"
+		  if ![TkFOR, TkWHILE, TkUNTIL].include?(@indent_stack.last)
+		    @indent += 1
+		    @indent_stack.push token_c
+		  end
+		else
+		  @indent += 1
+		  @indent_stack.push token_c
+		end
+#		p @indent_stack
+	      end
+
 	    elsif DEINDENT_CLAUSE.include?(token)
 	      @indent -= 1
+	      @indent_stack.pop
 	    end
 	    @lex_state = trans[0]
 	  else
@@ -752,6 +880,7 @@ class RubyLex
 
   def identify_here_document
     ch = getc
+#    if lt = PERCENT_LTYPE[ch]
     if ch == "-"
       ch = getc
       indent = true
@@ -783,7 +912,26 @@ class RubyLex
     end
 
     @here_header = false
-    while (l = gets.chomp) && (indent ? l.strip : l) != quoted
+#     while l = gets
+#       l = l.sub(/(:?\r)?\n\z/, '')
+#       if (indent ? l.strip : l) == quoted
+#  	break
+#       end
+#     end
+
+    line = ""
+    while ch = getc
+      if ch == "\n"
+	if line == quoted
+	  break
+	end
+	line = ""
+      else
+	line.concat ch unless indent && line == "" && /\s/ =~ ch
+	if @ltype != "'" && ch == "#" && peek(0) == "{"
+	  identify_string_dvar
+	end
+      end
     end
 
     @here_header = true
@@ -796,7 +944,7 @@ class RubyLex
     @lex_state = EXPR_END
     Token(Ltype2Token[lt])
   end
-  
+
   def identify_quotation
     ch = getc
     if lt = PERCENT_LTYPE[ch]
@@ -818,66 +966,132 @@ class RubyLex
   def identify_number
     @lex_state = EXPR_END
 
-    if ch = getc
-      if peek(0) == "x"
+    if peek(0) == "0" && peek(1) !~ /[.eE]/
+      getc
+      case peek(0)
+      when /[xX]/
+	ch = getc
+	match = /[0-9a-fA-F_]/
+      when /[bB]/
+	ch = getc
+	match = /[01_]/
+      when /[oO]/
 	ch = getc
-	match = /[0-9a-f_]/
-      else
 	match = /[0-7_]/
+      when /[dD]/
+	ch = getc
+	match = /[0-9_]/
+      when /[0-7]/
+	match = /[0-7_]/
+      when /[89]/
+	RubyLex.fail SyntaxError, "Invalid octal digit"
+      else
+	return Token(TkINTEGER)
       end
+
+      len0 = true
+      non_digit = false
       while ch = getc
-	if ch !~ match
+	if match =~ ch
+	  if ch == "_"
+	    if non_digit
+	      RubyLex.fail SyntaxError, "trailing `#{ch}' in number"
+	    else
+	      non_digit = ch
+	    end
+	  else
+	    non_digit = false
+	    len0 = false
+	  end
+	else
 	  ungetc
+	  if len0
+	    RubyLex.fail SyntaxError, "numeric literal without digits"
+	  end
+	  if non_digit
+	    RubyLex.fail SyntaxError, "trailing `#{non_digit}' in number"
+	  end
 	  break
 	end
       end
       return Token(TkINTEGER)
     end
-    
+
     type = TkINTEGER
-    allow_point = TRUE
-    allow_e = TRUE
+    allow_point = true
+    allow_e = true
+    non_digit = false
     while ch = getc
       case ch
-      when /[0-9_]/
+      when /[0-9]/
+	non_digit = false
+      when "_"
+	non_digit = ch
       when allow_point && "."
+	if non_digit
+	  RubyLex.fail SyntaxError, "trailing `#{non_digit}' in number"
+	end
 	type = TkFLOAT
 	if peek(0) !~ /[0-9]/
+	  type = TkINTEGER
 	  ungetc
 	  break
 	end
 	allow_point = false
       when allow_e && "e", allow_e && "E"
+	if non_digit
+	  RubyLex.fail SyntaxError, "trailing `#{non_digit}' in number"
+	end
 	type = TkFLOAT
 	if peek(0) =~ /[+-]/
 	  getc
 	end
 	allow_e = false
 	allow_point = false
+	non_digit = ch
       else
+	if non_digit
+	  RubyLex.fail SyntaxError, "trailing `#{non_digit}' in number"
+	end
 	ungetc
 	break
       end
     end
     Token(type)
   end
-  
+
   def identify_string(ltype, quoted = ltype)
     @ltype = ltype
     @quoted = quoted
     subtype = nil
     begin
-      while ch = getc 
-	if @quoted == ch
+      nest = 0
+      while ch = getc
+	if @quoted == ch and nest == 0
 	  break
-	elsif @ltype != "'" && @ltype != "]" and ch == "#"
+	elsif @ltype != "'" && ch == "#" && peek(0) == "{"
+	  identify_string_dvar
+	elsif @ltype != "'" && @ltype != "]" && @ltype != ":" and ch == "#"
 	  subtype = true
+	elsif ch == '\\' and @ltype == "'" #'
+	  case ch = getc
+	  when "\\", "\n", "'"
+	  else
+	    ungetc
+	  end
 	elsif ch == '\\' #'
 	  read_escape
 	end
+	if PERCENT_PAREN.values.include?(@quoted)
+	  if PERCENT_PAREN[ch] == @quoted
+	    nest += 1
+	  elsif ch == @quoted
+	    nest -= 1
+	  end
+	end
       end
       if @ltype == "/"
-	if peek(0) =~ /i|o|n|e|s/
+        while /[imxoesun]/ =~ peek(0)
 	  getc
 	end
       end
@@ -892,14 +1106,50 @@ class RubyLex
       @lex_state = EXPR_END
     end
   end
-  
+
+  def identify_string_dvar
+    begin
+      getc
+
+      reserve_continue = @continue
+      reserve_ltype = @ltype
+      reserve_indent = @indent
+      reserve_indent_stack = @indent_stack
+      reserve_state = @lex_state
+      reserve_quoted = @quoted
+
+      @ltype = nil
+      @quoted = nil
+      @indent = 0
+      @indent_stack = []
+      @lex_state = EXPR_BEG
+
+      loop do
+	@continue = false
+	prompt
+	tk = token
+	if @ltype or @continue or @indent > 0
+	  next
+	end
+	break if tk.kind_of?(TkRBRACE)
+      end
+    ensure
+      @continue = reserve_continue
+      @ltype = reserve_ltype
+      @indent = reserve_indent
+      @indent_stack = reserve_indent_stack
+      @lex_state = reserve_state
+      @quoted = reserve_quoted
+    end
+  end
+
   def identify_comment
     @ltype = "#"
 
     while ch = getc
-      if ch == "\\" #"
-	read_escape
-      end
+#      if ch == "\\" #"
+#	read_escape
+#      end
       if ch == "\n"
 	@ltype = nil
 	ungetc
@@ -908,11 +1158,11 @@ class RubyLex
     end
     return Token(TkCOMMENT)
   end
-  
+
   def read_escape
     case ch = getc
     when "\n", "\r", "\f"
-    when "\\", "n", "t", "r", "f", "v", "a", "e", "b" #"
+    when "\\", "n", "t", "r", "f", "v", "a", "e", "b", "s" #"
     when /[0-7]/
       ungetc ch
       3.times do
@@ -925,7 +1175,7 @@ class RubyLex
 	  break
 	end
       end
-      
+
     when "x"
       2.times do
 	case ch = getc
@@ -943,18 +1193,19 @@ class RubyLex
 	ungetc
       else
 	if (ch = getc) == "\\" #"
-	  read_escape(chrs)
+	  read_escape
 	end
       end
 
-    when "C", "c", "^"
+    when "C", "c" #, "^"
       if ch == "C" and (ch = getc) != "-"
 	ungetc
       elsif (ch = getc) == "\\" #"
-	read_escape(chrs)
+	read_escape
       end
     else
-      # other characters 
+      # other characters
     end
   end
 end
+# :startdoc:
