@@ -1,9 +1,9 @@
-require 'rdoc/class_module'
-
 ##
 # An anonymous class like:
 #
 #   c = Class.new do end
+#
+# AnonClass is currently not used.
 
 class RDoc::AnonClass < RDoc::ClassModule
 end
